/*
* Copyright © 2021 NVIDIA CORPORATION & AFFILIATES. ALL RIGHTS RESERVED.
*
* This software product is a proprietary product of Nvidia Corporation and its affiliates
* (the "Company") and all right, title, and interest in and to the software
* product, including all associated intellectual property rights, are and
* shall remain exclusively with the Company.
*
* This software product is governed by the End User License Agreement
* provided with the software product.
*/

#include <stdint.h>
#include <stdio.h>
#include <stdlib.h>

#include <netlink/cli/link.h>
#include <netlink/cli/utils.h>
#include <netlink/route/bridge_vlan.h>


int main(int argc, char *argv[])
{
	struct nl_cache *link_cache, *bv_cache;
	struct rtnl_link *needle, *match;
	static struct nl_sock *sock;
	int state, retval;
	uint16_t vid;
	char *endptr;

	if (argc != 3) {
		fprintf(stderr, "Usage: %s <ifname> <vid>\n", argv[0]);
		exit(2);
	}

	sock = nl_cli_alloc_socket();
	nl_cli_connect(sock, NETLINK_ROUTE);

	link_cache = nl_cli_link_alloc_cache(sock);
	needle = nl_cli_link_alloc();
	nl_cli_link_parse_name(needle, argv[1]);
	match = (struct rtnl_link *)nl_cache_find(link_cache,
						  OBJ_CAST(needle));
	rtnl_link_put(needle);
	nl_cache_mngt_unprovide(link_cache);
	nl_cache_free(link_cache);
	if (!match) {
		fprintf(stderr, "Error: interface \"%s\" not found\n", argv[1]);
		exit(1);
	}

	vid = strtoul(argv[2], &endptr, 0);
	if (*endptr != '\0') {
		fprintf(stderr, "Error: could not interpret vid \"%s\" as an integer\n",
			argv[optind]);
		exit(1);
	}

	retval = rtnl_bridge_vlan_alloc_cache(sock, &bv_cache);
	if (retval < 0) {
		nl_cli_fatal(retval, "Error: unable to allocate bridge_vlan cache: %s",
			     nl_geterror(retval));
	}
	nl_cache_mngt_provide(bv_cache);

	state = rtnl_link_bridge_vlan_get_port_state(match, vid);
	rtnl_link_put(match);
	if (state < 0) {
		nl_cli_fatal(state, "Error: unable to get vlan state: %s",
			     nl_geterror(state));
	}
	printf("%s\n", rtnl_bridge_vlan_port_states[state]);

	nl_cache_mngt_unprovide(bv_cache);
	nl_cache_free(bv_cache);
	nl_socket_free(sock);

	return 0;
}
